@@ -32,7 +32,6 @@ srm_context_t srm_context_new(const char * srm_endpoint, char *errbuf,int errbuf
 
 srm_context_t srm_context_new2(const char * srm_endpoint, char *errbuf,int errbufsz,int verbose, int keep_alive);
 
-
 /**
   free resources used by a srm context
 */
@@ -46,6 +45,16 @@ void srm_context_init(struct srm_context *context,char *srm_endpoint,char *errbu
 void srm_context_init2(struct srm_context *context,char *srm_endpoint,char *errbuf,int errbufsz,int verbose,
                        int keep_alive);
 
+/**
+ * set user credentials
+ */
+void srm_set_credentials(struct srm_context *context, const char *ucert, const char *ukey);
+
+/**
+ * Set total desired request time. For instance, for bringonline
+ */
+void srm_set_desired_request_time(struct srm_context *context, int timeout);
+
 /* srm_ls functions */
 void srm_ls_output_destroy(struct srm_ls_output *output);
 
@@ -120,6 +129,11 @@ int srm_getspacemd (struct srm_context *context,
 int srm_ping(struct srm_context *context,
 		struct srm_ping_output *output);
 
+int srm_xping(struct srm_context *context,
+        struct srm_xping_output *output);
+
+int srm_xping_output_free(struct srm_xping_output output);
+
 int srm_getpermission (struct srm_context *context,
 		struct srm_getpermission_input *input,struct srm_getpermission_output *output);
 
