@@ -16,7 +16,7 @@
  *
  * Authors: Todor Manev  IT-GT CERN
  */
- 
+
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -32,7 +32,7 @@
 
 #ifdef CMAKE_BUILD
 #define namespaces_srmv2 namespaces
-#endif 
+#endif
 
 const char *err_msg_begin = "SE";
 
@@ -100,7 +100,7 @@ void srm_set_timeout_sendreceive (int value)
 
 
 
-void back_off_logic_init(struct srm_context *context,struct srm_internal_context *internal_context)
+void back_off_logic_init(struct srm_context *context, struct srm_internal_context *internal_context)
 {
     GFAL_SRM_IFCE_ASSERT(context);
     GFAL_SRM_IFCE_ASSERT(internal_context);
@@ -134,70 +134,47 @@ void set_estimated_wait_time(struct srm_internal_context *internal_context, int
     internal_context->estimated_wait_time = (my_time > 0 && my_time < internal_context->relative_timeout)?my_time:-1;
 }
 
-void srm_soap_init(struct soap *soap)
+
+void srm_context_soap_init(struct srm_context* c)
 {
-	#ifdef GFAL_SECURE
-	int flags;
-	#endif
-	soap_init (soap);
-	soap->namespaces = namespaces_srmv2;
+    assert(c);
+    if (c->soap)
+        return;
 
-	#ifdef GFAL_SECURE
-	flags = CGSI_OPT_DISABLE_NAME_CHECK;
-	soap_register_plugin_arg (soap, client_cgsi_plugin, &flags);
-	#endif
-
-	soap->send_timeout =  srm_get_timeout_sendreceive ();
-	soap->recv_timeout =  srm_get_timeout_sendreceive ();
-	soap->connect_timeout = srm_get_timeout_connect ();
-}
-
-struct soap * srm_soap_init_new()
-{
-    return srm_soap_init_context_new(NULL);
-}
-
-struct soap * srm_soap_init_context_new(struct srm_context* c){
     #ifdef GFAL_SECURE
     int flags;
     #endif
-    struct soap *soap_handle = NULL;
 
-    if (c && c->ext && c->ext->keep_alive) {
-        soap_handle = soap_new2(SOAP_IO_KEEPALIVE, SOAP_IO_KEEPALIVE);
-        soap_handle->bind_flags |= SO_REUSEADDR;        
-        soap_handle->accept_timeout = 0;
-	soap_handle->tcp_keep_alive = 1;
-        soap_handle->socket_flags = MSG_NOSIGNAL;
+    if (c->ext && c->ext->keep_alive) {
+        c->soap = soap_new2(SOAP_IO_KEEPALIVE, SOAP_IO_KEEPALIVE);
+        c->soap->bind_flags |= SO_REUSEADDR;
+        c->soap->tcp_keep_alive = 1;
+        c->soap->socket_flags = MSG_NOSIGNAL;
     }
     else {
-        soap_handle = soap_new();
+        c->soap = soap_new();
     }
-   
-    soap_handle->namespaces = namespaces_srmv2;
+
+    c->soap->namespaces = namespaces_srmv2;
 
     #ifdef GFAL_SECURE
     flags = CGSI_OPT_DISABLE_NAME_CHECK | CGSI_OPT_KEEP_ALIVE;
-    soap_register_plugin_arg (soap_handle, client_cgsi_plugin, &flags);
+    soap_register_plugin_arg (c->soap, client_cgsi_plugin, &flags);
+    if (c->ext) {
+        if (cgsi_plugin_set_credentials(c->soap, 0, c->ext->ucert, c->ext->ukey) != 0) {
+            soap_free(c->soap);
+            c->soap = NULL;
+            srm_errmsg (c, "[SRM][srm_soap_init_context_new] could not load client credentials");
+            return;
+        }
+    }
     #endif
 
-    if(c){
-        soap_handle->recv_timeout= c->timeout_ops;
-        soap_handle->send_timeout = c->timeout_ops;
-        soap_handle->connect_timeout = c->timeout_conn;
-    }else{
-        soap_handle->send_timeout =  srm_get_timeout_sendreceive ();
-        soap_handle->recv_timeout =  srm_get_timeout_sendreceive ();
-        soap_handle->connect_timeout = srm_get_timeout_connect ();
-    }
-    return soap_handle;
+    c->soap->recv_timeout= c->timeout_ops;
+    c->soap->send_timeout = c->timeout_ops;
+    c->soap->connect_timeout = c->timeout_conn;
 }
 
-void srm_soap_deinit(struct soap *soap)
-{
-	soap_end (soap);
-	soap_done (soap);
-}
 
 void srm_soap_free(struct soap *soap)
 {
@@ -223,6 +200,7 @@ int statuscode2errno (int statuscode)
 	switch (statuscode) {
 		case SRM_USCOREINVALID_USCOREPATH:
 			return (ENOENT);
+		case SRM_USCOREAUTHENTICATION_USCOREFAILURE:
 		case SRM_USCOREAUTHORIZATION_USCOREFAILURE:
 			return (EACCES);
 		case SRM_USCOREDUPLICATION_USCOREERROR:
@@ -235,27 +213,48 @@ int statuscode2errno (int statuscode)
 			return (EOPNOTSUPP);
 		case SRM_USCORENON_USCOREEMPTY_USCOREDIRECTORY:
 			return (ENOTEMPTY);
+		case SRM_USCOREFILE_USCORELIFETIME_USCOREEXPIRED:
+			return (ETIME);
+		case SRM_USCOREREQUEST_USCOREINPROGRESS:
+		case SRM_USCOREREQUEST_USCOREQUEUED:
+			return (EAGAIN);
+		case SRM_USCOREFAILURE:
+			return (EIO);
+		case SRM_USCORENO_USCOREUSER_USCORESPACE:
+		case SRM_USCOREINVALID_USCOREREQUEST:
+			return (EBADR);
+		case SRM_USCOREEXCEED_USCOREALLOCATION:
+			return (EDQUOT);
+		case SRM_USCOREFILE_USCOREBUSY:
+		case SRM_USCOREFILE_USCOREUNAVAILABLE:
+			return (EBUSY);
+		case SRM_USCOREFILE_USCORELOST:
+			return (EIDRM);
+		case SRM_USCOREABORTED:
+			return (ECANCELED);
 		case SRM_USCORESUCCESS:
 		case SRM_USCOREFILE_USCOREPINNED:
 		case SRM_USCORESPACE_USCOREAVAILABLE:
+		case SRM_USCORERELEASED:
 			return (0);
 		default:
 			return (EINVAL);
 	}
 }
 
-int statuscode_and_msg_to_errno(int statuscode, const char** err_tab_msg){
+int statuscode_and_msg_to_errno(int statuscode, const char* err_tab_msg){
     int err_code = statuscode2errno(statuscode);
-    if(err_code == EINVAL && err_tab_msg != NULL){
+    if((err_code == EINVAL || err_code == EIO) && err_tab_msg != NULL){
         // search for string pattern
-        char** err_tab = (char**) err_tab_msg;
-        while(*err_tab != NULL){
-            // search for pattern EEXIST, LCGUTIL-203 bug
-            if( strstr(*err_tab,"exists, overwite is not allowed") != NULL)
-                err_code = EEXIST;
-
-            ++err_tab;
-        }
+		// search for pattern EEXIST, LCGUTIL-203 bug
+		if( strstr(err_tab_msg, "exists, overwite is not allowed") != NULL)
+			err_code = EEXIST;
+		// https://its.cern.ch/jira/browse/DMC-512
+		if (strstr(err_tab_msg, "o such file or directory") != NULL)
+			err_code = ENOENT;
+		// https://its.cern.ch/jira/browse/DMC-528
+		if (strstr(err_tab_msg, "ile does not exist") != NULL)
+			err_code = ENOENT;
     }
     return err_code;
 }
@@ -272,7 +271,7 @@ void srm_errmsg (struct srm_context *context, const char *format, ...)
 
 	va_start (ap, format);
      (void)asprintf (&actual_format, "%s\n", format);
-	if (actual_format == NULL){ 
+	if (actual_format == NULL){
 		va_end(ap);
 		return;
 	}
@@ -285,23 +284,25 @@ void srm_errmsg (struct srm_context *context, const char *format, ...)
 	free (actual_format);
 	va_end(ap);
 }
-int srm_soap_call_err(struct srm_context *context,struct soap *soap,const char *srmfunc)
+
+int srm_soap_call_err(struct srm_context *context, const char *srmfunc)
 {
-	if (soap->fault != NULL && soap->fault->faultstring != NULL)
+	if (context->soap->fault != NULL && context->soap->fault->faultstring != NULL)
 	{
 		srm_errmsg (context, "[%s][%s][] %s: %s",err_msg_begin,
-				 srmfunc, context->srm_endpoint, soap->fault->faultstring);
-	}else if (soap->error == SOAP_EOF)
+				 srmfunc, context->srm_endpoint, context->soap->fault->faultstring);
+	}else if (context->soap->error == SOAP_EOF)
 	{
 		srm_errmsg (context, "[%s][%s][] %s: Connection fails or timeout",
 				err_msg_begin,srmfunc,context->srm_endpoint);
 	}else
 	{
 		srm_errmsg (context, "[%s][%s][] %s: Unknown SOAP error (%d)",
-				err_msg_begin,srmfunc,context->srm_endpoint, soap->error);
+				err_msg_begin,srmfunc,context->srm_endpoint, context->soap->error);
 	}
 	return ECOMM;
 }
+
 int srm_call_err(struct srm_context *context,struct srm2__TReturnStatus  *retstatus,
 		const char *srmfunc)
 {
@@ -317,8 +318,7 @@ int srm_call_err(struct srm_context *context,struct srm2__TReturnStatus  *retsta
 			retstatus->statusCode != SRM_USCOREDONE &&
 			retstatus->statusCode != SRM_USCORETOO_USCOREMANY_USCORERESULTS)
 	{
-        const char * tab_err_msg[] = { retstatus->explanation, NULL };
-        result_errno = statuscode_and_msg_to_errno(retstatus->statusCode, tab_err_msg);
+        result_errno = statuscode_and_msg_to_errno(retstatus->statusCode, retstatus->explanation);
 		if (retstatus->explanation && retstatus->explanation[0])
 		{
 			srm_errmsg (context, "[%s][%s][%s] %s: %s",err_msg_begin,
@@ -340,6 +340,7 @@ int srm_call_err(struct srm_context *context,struct srm2__TReturnStatus  *retsta
 	}
 	return result_errno;
 }
+
 // return error status
 int srm_print_error_status(struct srm_context *context,struct srm2__TReturnStatus *status,const char *srmfunc)
 {
@@ -356,6 +357,7 @@ int srm_print_error_status(struct srm_context *context,struct srm2__TReturnStatu
 	}
 	return statuscode2errno (status->statusCode);
 }
+
 int srm_print_error_status_additional(struct srm_context *context,struct srm2__TReturnStatus *status,const char *srmfunc,char *additional_info)
 {
 
@@ -393,7 +395,7 @@ int wait_for_new_attempt_min_max_ng(srm_context_t context, struct srm_internal_c
     if( timespec_cmp(&end_sleep_time, &(internal_context->end_time_spec), <) ){
         //printf(" I wil sleep %ld %ld", internal_context->current_waittime_spec.tv_sec, internal_context->current_waittime_spec.tv_nsec);
         while(timespec_cmp(&(current_time), &end_sleep_time, <)){ // go to sleep for the exact required time
-            call_function.call_usleep(1);
+            call_function.call_usleep(500000);
             clock_gettime(CLOCK_MONOTONIC, &current_time);
         }
 
@@ -435,8 +437,8 @@ int wait_for_new_attempt(struct srm_internal_context *internal_context)  // Or T
 			if (after_sleep >= internal_context->end_time)
 			{
                 wait_till_end = internal_context->end_time - time(NULL) -  last_chance_sec_before_end; //try the last hope before the end
-				if (wait_till_end>0) 
-					call_function.call_sleep(wait_till_end); 
+				if (wait_till_end>0)
+					call_function.call_sleep(wait_till_end);
                 else // deadline outdated,
                     return -1; 					// simply return in timeout
 			}else
@@ -546,7 +548,6 @@ void srm_spacemd_free (int nbtokens,struct srm_spacemd *smd)
 }
 int srm_set_protocol_in_transferParameters(
 	struct srm_context *context,
-    struct soap* soap,
     struct srm2__TTransferParameters* transferParameters,
     char** protocols)
 {
@@ -556,7 +557,7 @@ int srm_set_protocol_in_transferParameters(
     *array = NULL;
 
     if (protocols) {
-        *array = soap_malloc (soap, sizeof(struct srm2__ArrayOfString));
+        *array = soap_malloc (context->soap, sizeof(struct srm2__ArrayOfString));
 
         if (*array == NULL) {
             srm_errmsg (context, "[SRM][soap_malloc][] error");
@@ -626,6 +627,7 @@ int copy_string(char **dest,char *src)
 	}
 	return 0;
 }
+
 int copy_permissionfilestatuses(struct srm2__TReturnStatus *reqstatp,
 						struct srmv2_filestatus **statuses,
 						struct srm2__ArrayOfTSURLPermissionReturn *repfs,
@@ -1036,7 +1038,7 @@ void copy_Locality(struct srm2__TMetaDataPathDetail *soap_file_meta_data, struct
 			break;
 		case UNAVAILABLE:
 			res_loc = GFAL_LOCALITY_UNAVAILABLE;
-			break;			
+			break;
 		default:
 			res_loc = GFAL_LOCALITY_UNKNOWN;
 	}
@@ -1127,12 +1129,14 @@ int copy_mdfilestatuses(struct srm2__TReturnStatus *reqstatp,
 		if (repfs->pathDetailArray[i]->createdAtTime)
 		{
 			struct tm createdAtTime;
+			memset(&createdAtTime, 0, sizeof(createdAtTime));
 			strptime(repfs->pathDetailArray[i]->createdAtTime, "%Y-%m-%dT%T", &createdAtTime);
 			(*statuses)[i].stat.st_ctime = mktime(&createdAtTime);
 		}
 		if (repfs->pathDetailArray[i]->lastModificationTime)
 		{
 			struct tm modifiedTime;
+			memset(&modifiedTime, 0, sizeof(modifiedTime));
 			strptime(repfs->pathDetailArray[i]->lastModificationTime, "%Y-%m-%dT%T", &modifiedTime);
 			(*statuses)[i].stat.st_mtime = mktime(&modifiedTime);
 		}
@@ -1229,7 +1233,7 @@ char* srm_util_consolidate_multiple_characters(const char* s, const char c, cons
 
     if (s == 0) {
         return 0;
-    }    
+    }
 
     tmp = (char*) malloc (strlen(s) + 1);
 
@@ -1238,12 +1242,12 @@ char* srm_util_consolidate_multiple_characters(const char* s, const char c, cons
         if (i < start || s[i] != c || s[i + 1] != c) {
             tmp[tmp_i] = s[i];
             ++tmp_i;
-        } 
-    } 
-    
+        }
+    }
+
     tmp[tmp_i] = 0;
     // strdup the string, to shrink to the real size
-    ret = strdup(tmp);    
+    ret = strdup(tmp);
     free(tmp);
     return ret;
 }
@@ -1256,7 +1260,7 @@ char* srm_util_add_strings(const char* s1, const char* s2)
 
     assert(s1);
     assert(s2);
-   
+
     if (!s1 || !s2) {
         return 0;
     }
@@ -1264,7 +1268,7 @@ char* srm_util_add_strings(const char* s1, const char* s2)
     len_s1 = strlen(s1);
     ret = malloc(len_s1 + strlen(s2) + 1);
     assert(ret);
-   
+
     if (ret) {
         strcpy(ret, s1);
         strcat(ret + len_s1, s2);
