@@ -42,11 +42,12 @@ int srmv2_status_of_put_request_async_internal(struct srm_context *context,
 {
 	struct srm2__ArrayOfTPutRequestFileStatus *repfs;
 	int ret = 0;
-    struct soap* soap = srm_soap_init_context_new(context);
 	struct srm2__srmStatusOfPutRequestResponse_ srep;
 	struct srm2__srmStatusOfPutRequestRequest sreq;
 	const char srmfunc[] = "StatusOfPutRequest";
 
+	srm_context_soap_init(context);
+
 	memset (&sreq, 0, sizeof(sreq));
 	sreq.requestToken = output->token;
 	output->retstatus = NULL;
@@ -54,14 +55,13 @@ int srmv2_status_of_put_request_async_internal(struct srm_context *context,
 
 	do
 	{
-		ret = call_function.call_srm2__srmStatusOfPutRequest(soap, context->srm_endpoint, srmfunc, &sreq, &srep);
+		ret = call_function.call_srm2__srmStatusOfPutRequest(context->soap, context->srm_endpoint, srmfunc, &sreq, &srep);
 		// If no response break with failure
 		if ((srep.srmStatusOfPutRequestResponse == NULL)||(ret!=0)||
 				copy_returnstatus(&output->retstatus,srep.srmStatusOfPutRequestResponse->returnStatus))
 		{
-			errno = srm_soap_call_err(context,soap,srmfunc);
+			errno = srm_soap_call_err(context, srmfunc);
 			internal_context->current_status = srm_call_status_FAILURE;
-			srm_soap_free(soap);
 			return -1;
 		}
         set_estimated_wait_time(internal_context, srmv2_put_status_estimated_wait_time(&srep));
@@ -74,9 +74,10 @@ int srmv2_status_of_put_request_async_internal(struct srm_context *context,
 
 
 	if (output->retstatus->statusCode == SRM_USCORESPACE_USCORELIFETIME_USCOREEXPIRED) {
-		srm_errmsg (context,"[SRM][%s][%s] %s: Space lifetime expired",srmfunc, statuscode2errmsg(output->retstatus->statusCode), context->srm_endpoint);
+		srm_errmsg(context, "[SRM][%s][%s] %s: Space lifetime expired", srmfunc,
+				statuscode2errmsg(output->retstatus->statusCode),
+				context->srm_endpoint);
 		errno = statuscode2errno(output->retstatus->statusCode);
-		srm_soap_free(soap);
 		return (-1);
 	}
 
@@ -110,7 +111,6 @@ int srmv2_status_of_put_request_async_internal(struct srm_context *context,
 			ret = -1;
 			break;
 	}
-	srm_soap_free(soap);
 	return ret;
 }
 
@@ -138,45 +138,35 @@ int srmv2_prepare_to_put_async_internal(struct srm_context *context,
 	struct srm2__srmPrepareToPutResponse_ rep;
 	struct srm2__ArrayOfTPutRequestFileStatus *repfs;
 	struct srm2__srmPrepareToPutRequest req;
-    struct soap* soap = srm_soap_init_context_new(context);
 	static enum srm2__TFileStorageType s_types[] = {VOLATILE, DURABLE, PERMANENT};
 	char *targetspacetoken;
 	const char srmfunc[] = "PrepareToPut";
 	struct srm_getbestspacetokens_input spacetokeninput;
 	SRM_LONG64 totalsize=0;
 
+	srm_context_soap_init(context);
 
-	memset (&req, 0, sizeof(req));
-	memset (&rep, 0, sizeof(rep));
+	memset(&req, 0, sizeof(req));
+	memset(&rep, 0, sizeof(rep));
 	memset(output,0,sizeof(*output));
 
 	if ((req.arrayOfFileRequests =
-				soap_malloc (soap, sizeof(struct srm2__ArrayOfTPutFileRequest))) == NULL ||
+				soap_malloc (context->soap, sizeof(struct srm2__ArrayOfTPutFileRequest))) == NULL ||
 			(req.arrayOfFileRequests->requestArray =
-			 soap_malloc (soap, input->nbfiles* sizeof(struct srm2__TPutFileRequest *))) == NULL ||
+			 soap_malloc (context->soap, input->nbfiles* sizeof(struct srm2__TPutFileRequest *))) == NULL ||
 			(req.transferParameters =
-			 soap_malloc (soap, sizeof(struct srm2__TTransferParameters))) == NULL ||
-			(req.targetSpaceToken =
-			 soap_malloc (soap, sizeof(char *))) == NULL) {
+			 soap_malloc (context->soap, sizeof(struct srm2__TTransferParameters))) == NULL) {
 
 		srm_errmsg (context, "[SRM][soap_malloc][] error");
 		errno = ENOMEM;
-		srm_soap_free(soap);
 		return (-1);
 	}
 
-    if(context->ext && context->ext->turl_timeout){ // advanced context -> turl timeout
-        req.desiredPinLifeTime = &(context->ext->turl_timeout);
-        req.desiredTotalRequestTime = &(context->ext->turl_timeout);
-    }else{
-        if (input->desiredpintime > 0)
-            req.desiredPinLifeTime = &input->desiredpintime;
-        else
-            req.desiredPinLifeTime = &context->timeout_ops;
+	if (input->desiredpintime > 0)
+		req.desiredPinLifeTime = &input->desiredpintime;
 
-        if (context->timeout_ops > 0)
-            req.desiredTotalRequestTime = &context->timeout_ops;
-    }
+	if (context->ext && context->ext->turl_resolution_timeout)
+		req.desiredTotalRequestTime = &(context->ext->turl_resolution_timeout);
 
 	req.desiredFileStorageType = &s_types[PERMANENT];
 	req.arrayOfFileRequests->__sizerequestArray = input->nbfiles;
@@ -184,25 +174,22 @@ int srmv2_prepare_to_put_async_internal(struct srm_context *context,
 	if (input->filesizes == NULL &&  input->nbfiles>0)
 	{
 		errno = EINVAL;
-		srm_soap_free(soap);
 		return (-1);
 	}
 
 	for (i = 0; i < input->nbfiles; i++) {
 		if ((req.arrayOfFileRequests->requestArray[i] =
-					soap_malloc (soap, sizeof(struct srm2__TPutFileRequest))) == NULL) {
+					soap_malloc (context->soap, sizeof(struct srm2__TPutFileRequest))) == NULL) {
 			srm_errmsg (context, "[SRM][soap_malloc][] error");
 			errno = ENOMEM;
-			srm_soap_free(soap);
 			return (-1);
 		}
 		memset (req.arrayOfFileRequests->requestArray[i], 0, sizeof(struct srm2__TPutFileRequest));
 		req.arrayOfFileRequests->requestArray[i]->targetSURL = (char *)input->surls[i];
 
-		if ((req.arrayOfFileRequests->requestArray[i]->expectedFileSize = soap_malloc (soap, sizeof(ULONG64))) == NULL) {
+		if ((req.arrayOfFileRequests->requestArray[i]->expectedFileSize = soap_malloc (context->soap, sizeof(ULONG64))) == NULL) {
 			srm_errmsg (context, "[SRM][soap_malloc][] error");
 			errno = ENOMEM;
-			srm_soap_free(soap);
 			return (-1);
 		}
 		*(req.arrayOfFileRequests->requestArray[i]->expectedFileSize) = input->filesizes[i];
@@ -214,11 +201,10 @@ int srmv2_prepare_to_put_async_internal(struct srm_context *context,
 	req.transferParameters->arrayOfClientNetworks = NULL;
 	req.transferParameters->arrayOfTransferProtocols = NULL;
 
-	ret = srm_set_protocol_in_transferParameters(context,soap, req.transferParameters,input->protocols);
+	ret = srm_set_protocol_in_transferParameters(context, req.transferParameters,input->protocols);
 
 	if (ret)
 	{
-		srm_soap_free(soap);
 		return (-1);
 	}
 
@@ -237,7 +223,6 @@ int srmv2_prepare_to_put_async_internal(struct srm_context *context,
 				req.targetSpaceToken = targetspacetoken;
 			} else
 			{
-				srm_soap_free(soap);
 				return (-1);
 			}
 	}
@@ -268,14 +253,13 @@ int srmv2_prepare_to_put_async_internal(struct srm_context *context,
 
 	do
 	{
-		ret = call_function.call_srm2__srmPrepareToPut (soap, context->srm_endpoint, srmfunc, &req, &rep);
+		ret = call_function.call_srm2__srmPrepareToPut (context->soap, context->srm_endpoint, srmfunc, &req, &rep);
 		// If no response break with failure
 		if ((rep.srmPrepareToPutResponse== NULL)||(ret!=0)||
 				copy_returnstatus(&output->retstatus,rep.srmPrepareToPutResponse->returnStatus))
 		{
-			errno = srm_soap_call_err(context,soap,srmfunc);
+			errno = srm_soap_call_err(context, srmfunc);
 			internal_context->current_status = srm_call_status_FAILURE;
-			srm_soap_free(soap);
 			return -1;
 		}
         set_estimated_wait_time(internal_context, srmv2_put_estimated_wait_time(&rep));
@@ -298,10 +282,7 @@ int srmv2_prepare_to_put_async_internal(struct srm_context *context,
 			break;
 		case srm_call_status_SUCCESS:
 		case srm_call_status_FAILURE:
-			if (!repfs ||
-                repfs->__sizestatusArray < 1 ||
-                !repfs->statusArray ||
-                internal_context->current_status == srm_call_status_FAILURE)
+			if (!repfs || repfs->__sizestatusArray < 1 || !repfs->statusArray || internal_context->current_status == srm_call_status_FAILURE)
 			{
 			    internal_context->current_status = srm_call_status_FAILURE;
 
@@ -321,13 +302,13 @@ int srmv2_prepare_to_put_async_internal(struct srm_context *context,
 					internal_context->current_status = srm_call_status_FAILURE;
 					errno = srm_call_err(context,output->retstatus,srmfunc);
 					ret = -1;
-				}else{				
+				}else{
 					errno = 0;
 					internal_context->current_status = srm_call_status_SUCCESS;
 					ret = copy_pinfilestatuses_put(output->retstatus, &output->filestatuses,
 												repfs,
 												srmfunc);
-				}				
+				}
 			}
 			break;
         case srm_call_status_TIMEOUT: // add timeout management for backoff logic
@@ -340,7 +321,6 @@ int srmv2_prepare_to_put_async_internal(struct srm_context *context,
 			break;
 	}
 
-	srm_soap_free(soap);
 	return ret;
 }
 
@@ -369,26 +349,23 @@ int srmv2_prepare_to_get_async_internal(struct srm_context *context,
 	struct srm2__ArrayOfTGetRequestFileStatus *repfs;
 	struct srm2__srmPrepareToGetRequest req;
 	struct srm_getbestspacetokens_input spacetokeninput;
-    struct soap* soap = srm_soap_init_context_new(context);
 	static enum srm2__TFileStorageType s_types[] = {VOLATILE, DURABLE, PERMANENT};
 	const char srmfunc[] = "PrepareToGet";
 
+	srm_context_soap_init(context);
 
 	/* issue "get" request */
 	memset (&req, 0, sizeof(req));
 	memset(output,0,sizeof(*output));
 	if ((req.arrayOfFileRequests =
-				soap_malloc (soap, sizeof(struct srm2__ArrayOfTGetFileRequest))) == NULL ||
+				soap_malloc (context->soap, sizeof(struct srm2__ArrayOfTGetFileRequest))) == NULL ||
 			(req.arrayOfFileRequests->requestArray =
-			 soap_malloc (soap, input->nbfiles * sizeof(struct srm2__TGetFileRequest *))) == NULL ||
+			 soap_malloc (context->soap, input->nbfiles * sizeof(struct srm2__TGetFileRequest *))) == NULL ||
 			(req.transferParameters =
-			 soap_malloc (soap, sizeof(struct srm2__TTransferParameters))) == NULL ||
-			(req.targetSpaceToken =
-			 soap_malloc (soap, sizeof(char *))) == NULL) {
+			 soap_malloc (context->soap, sizeof(struct srm2__TTransferParameters))) == NULL) {
 
 		srm_errmsg (context, "[SRM][soap_malloc][] error");
 		errno = ENOMEM;
-		srm_soap_free(soap);
 		return (-1);
 	}
 
@@ -406,24 +383,16 @@ int srmv2_prepare_to_get_async_internal(struct srm_context *context,
 			req.targetSpaceToken = targetspacetoken;
 		} else
 		{
-			srm_soap_free(soap);
 			return (-1);
 		}
 	}
 
 
-    if(context->ext && context->ext->turl_timeout){ // advanced context -> turl timeout
-        req.desiredPinLifeTime = &(context->ext->turl_timeout);
-        req.desiredTotalRequestTime = &(context->ext->turl_timeout);
-    }else{
-        if (input->desiredpintime > 0)
-            req.desiredPinLifeTime = &input->desiredpintime;
-        else
-            req.desiredPinLifeTime = &context->timeout;
+	if (input->desiredpintime > 0)
+		req.desiredPinLifeTime = &input->desiredpintime;
 
-        if (context->timeout_ops > 0)
-            req.desiredTotalRequestTime = &context->timeout_ops;
-    }
+	if (context->ext && context->ext->turl_resolution_timeout)
+		req.desiredTotalRequestTime = &(context->ext->turl_resolution_timeout);
 
 	req.desiredFileStorageType = &s_types[PERMANENT];
 	req.arrayOfFileRequests->__sizerequestArray = input->nbfiles;
@@ -431,11 +400,10 @@ int srmv2_prepare_to_get_async_internal(struct srm_context *context,
 	for (i = 0; i < input->nbfiles; i++)
 	{
 		if ((req.arrayOfFileRequests->requestArray[i] =
-					soap_malloc (soap, sizeof(struct srm2__TGetFileRequest))) == NULL)
+					soap_malloc (context->soap, sizeof(struct srm2__TGetFileRequest))) == NULL)
 		{
 			srm_errmsg (context, "[SRM][soap_malloc][] error");
 			errno = ENOMEM;
-			srm_soap_free(soap);
 			return (-1);
 		}
 		memset (req.arrayOfFileRequests->requestArray[i], 0, sizeof(struct srm2__TGetFileRequest));
@@ -447,24 +415,22 @@ int srmv2_prepare_to_get_async_internal(struct srm_context *context,
 	req.transferParameters->connectionType = NULL;
 	req.transferParameters->arrayOfClientNetworks = NULL;
 
-	ret = srm_set_protocol_in_transferParameters(context,soap, req.transferParameters,input->protocols);
+	ret = srm_set_protocol_in_transferParameters(context, req.transferParameters,input->protocols);
 
 	if (ret)
 	{
-		srm_soap_free(soap);
 		return (-1);
 	}
 
 	do
 	{
-		ret = call_function.call_srm2__srmPrepareToGet (soap, context->srm_endpoint, srmfunc, &req, &rep);
+		ret = call_function.call_srm2__srmPrepareToGet (context->soap, context->srm_endpoint, srmfunc, &req, &rep);
 		// If no response break with failure
 		if ((rep.srmPrepareToGetResponse== NULL)||(ret!=0)||
 				copy_returnstatus(&output->retstatus,rep.srmPrepareToGetResponse->returnStatus))
 		{
-			errno = srm_soap_call_err(context,soap,srmfunc);
+			errno = srm_soap_call_err(context, srmfunc);
 			internal_context->current_status = srm_call_status_FAILURE;
-			srm_soap_free(soap);
 			return -1;
 		}
         set_estimated_wait_time(internal_context, srmv2_get_estimated_wait_time(&rep));
@@ -510,13 +476,13 @@ int srmv2_prepare_to_get_async_internal(struct srm_context *context,
 				errno = 0;
 				internal_context->current_status = srm_call_status_SUCCESS;
                 if(rep.srmPrepareToGetResponse->requestToken != NULL){
-                    copy_string(&output->token,rep.srmPrepareToGetResponse->requestToken);
+                    ret =  copy_string(&output->token,rep.srmPrepareToGetResponse->requestToken);
                 }
-				ret = copy_pinfilestatuses_get(output->retstatus,
-											&output->filestatuses,
-											repfs,
-											srmfunc);
-
+                if (ret >= 0)
+                    ret = copy_pinfilestatuses_get(output->retstatus,
+                                                &output->filestatuses,
+                                                repfs,
+                                                srmfunc);
 			}
 			break;
         case srm_call_status_TIMEOUT: // add timeout management for backoff logic
@@ -529,7 +495,6 @@ int srmv2_prepare_to_get_async_internal(struct srm_context *context,
 			break;
 	}
 
-	srm_soap_free(soap);
 	return ret;
 }
 
@@ -554,11 +519,11 @@ int srmv2_status_of_get_request_async_internal(struct srm_context *context,
 {
 	struct srm2__ArrayOfTGetRequestFileStatus *repfs;
 	int ret = 0;
-    struct soap* soap = srm_soap_init_context_new(context);
 	struct srm2__srmStatusOfGetRequestResponse_ srep;
 	struct srm2__srmStatusOfGetRequestRequest sreq;
     const char srmfunc[] = "StatusOfGetRequest";
 
+    srm_context_soap_init(context);
 
 	memset (&sreq, 0, sizeof(sreq));
 	sreq.requestToken = output->token;
@@ -567,14 +532,13 @@ int srmv2_status_of_get_request_async_internal(struct srm_context *context,
 
 	do
 	{
-		ret = call_function.call_srm2__srmStatusOfGetRequest(soap, context->srm_endpoint, srmfunc, &sreq, &srep);
+		ret = call_function.call_srm2__srmStatusOfGetRequest(context->soap, context->srm_endpoint, srmfunc, &sreq, &srep);
 		// If no response break with failure
 		if ((srep.srmStatusOfGetRequestResponse == NULL)||(ret!=0) ||
 				copy_returnstatus(&output->retstatus,srep.srmStatusOfGetRequestResponse->returnStatus))
 		{
-			errno = srm_soap_call_err(context,soap,srmfunc);
+			errno = srm_soap_call_err(context, srmfunc);
 			internal_context->current_status = srm_call_status_FAILURE;
-			srm_soap_free(soap);
 			return -1;
 		}
         set_estimated_wait_time(internal_context, srmv2_get_status_estimated_wait_time(&srep));
@@ -628,10 +592,9 @@ int srmv2_status_of_get_request_async_internal(struct srm_context *context,
 			break;
 	}
 
-	srm_soap_free(soap);
 	return ret;
-
 }
+
 int srmv2_put_done(struct srm_context *context,
 		struct srm_putdone_input *input, struct srmv2_filestatus **statuses)
 {
@@ -641,9 +604,10 @@ int srmv2_put_done(struct srm_context *context,
     struct srm2__TReturnStatus *reqstatp;
 	struct srm2__ArrayOfTSURLReturnStatus *repfs;
 	struct srm2__srmPutDoneRequest req;
-    struct soap* soap = srm_soap_init_context_new(context);
 	const char srmfunc[] = "PutDone";
 
+	srm_context_soap_init(context);
+
 	// Setup the timeout
 	back_off_logic_init(context,&internal_context);
 
@@ -653,10 +617,9 @@ int srmv2_put_done(struct srm_context *context,
 
 	// NOTE: only one SURL in the array
 	if ((req.arrayOfSURLs =
-				soap_malloc (soap, sizeof(struct srm2__ArrayOfAnyURI))) == NULL) {
+				soap_malloc (context->soap, sizeof(struct srm2__ArrayOfAnyURI))) == NULL) {
 		srm_errmsg (context, "[SRM][soap_malloc][] error");
 		errno = ENOMEM;
-		srm_soap_free(soap);
 		return (-1);
 	}
 
@@ -666,13 +629,12 @@ int srmv2_put_done(struct srm_context *context,
 
 	do
 	{
-		ret = call_function.call_srm2__srmPutDone(soap, context->srm_endpoint, srmfunc, &req, &rep);
+		ret = call_function.call_srm2__srmPutDone(context->soap, context->srm_endpoint, srmfunc, &req, &rep);
 		// If no response break with failure
 		if ((rep.srmPutDoneResponse == NULL)||(ret!=0))
 		{
-			errno = srm_soap_call_err(context,soap,srmfunc);
+			errno = srm_soap_call_err(context, srmfunc);
 			internal_context.current_status = srm_call_status_FAILURE;
-			srm_soap_free(soap);
 			return -1;
 		}
 		// Check status and wait with back off logic if necessary(Internal_error)
@@ -693,7 +655,6 @@ int srmv2_put_done(struct srm_context *context,
             if (!repfs || repfs->__sizestatusArray < 1 || !repfs->statusArray)
             {
                 errno = srm_call_err(context,reqstatp,srmfunc);
-                srm_soap_free(soap);
                 return (-1);
             }else
             {
@@ -705,9 +666,9 @@ int srmv2_put_done(struct srm_context *context,
 
     }
 
-	srm_soap_free(soap);
 	return (ret);
 }
+
 int srmv2_release_files(struct srm_context *context,
 		struct srm_releasefiles_input *input, struct srmv2_filestatus **statuses)
 {
@@ -717,9 +678,10 @@ int srmv2_release_files(struct srm_context *context,
 	struct srm2__ArrayOfTSURLReturnStatus *repfs;
 	struct srm2__srmReleaseFilesRequest req;
 	struct srm2__TReturnStatus *reqstatp;
-    struct soap* soap = srm_soap_init_context_new(context);
 	const char srmfunc[] = "ReleaseFiles";
 
+	srm_context_soap_init(context);
+
 	// Setup the timeout
 	back_off_logic_init(context,&internal_context);
 
@@ -729,10 +691,9 @@ int srmv2_release_files(struct srm_context *context,
 
 	// NOTE: only one SURL in the array
 	if ((req.arrayOfSURLs =
-				soap_malloc (soap, sizeof(struct srm2__ArrayOfAnyURI))) == NULL) {
+				soap_malloc (context->soap, sizeof(struct srm2__ArrayOfAnyURI))) == NULL) {
 		srm_errmsg (context, "[SRM][soap_malloc][] error");
 		errno = ENOMEM;
-		srm_soap_free(soap);
 		return (-1);
 	}
 
@@ -741,13 +702,12 @@ int srmv2_release_files(struct srm_context *context,
 
 	do
 	{
-		ret = call_function.call_srm2__srmReleaseFiles(soap, context->srm_endpoint, srmfunc, &req, &rep);
+		ret = call_function.call_srm2__srmReleaseFiles(context->soap, context->srm_endpoint, srmfunc, &req, &rep);
 		// If no response break with failure
 		if ((rep.srmReleaseFilesResponse == NULL)||(ret!=0))
 		{
-			errno = srm_soap_call_err(context,soap,srmfunc);
+			errno = srm_soap_call_err(context, srmfunc);
 			internal_context.current_status = srm_call_status_FAILURE;
-			srm_soap_free(soap);
 			return -1;
 		}
 		// Copy response status
@@ -771,7 +731,6 @@ int srmv2_release_files(struct srm_context *context,
             if (!repfs || repfs->__sizestatusArray < 1 || !repfs->statusArray)
             {
                 errno = srm_call_err(context,reqstatp,srmfunc);
-                srm_soap_free(soap);
                 return (-1);
             }else
             {
@@ -782,7 +741,6 @@ int srmv2_release_files(struct srm_context *context,
             break;
     }
 
-	srm_soap_free(soap);
 	return (ret);
 }
 
@@ -811,29 +769,25 @@ int srmv2_bring_online_async_internal (struct srm_context *context,
 	struct srm2__srmBringOnlineResponse_ rep;
 	struct srm2__ArrayOfTBringOnlineRequestFileStatus *repfs;
 	struct srm2__srmBringOnlineRequest req;
-    struct soap* soap = srm_soap_init_context_new(context);
 	static enum srm2__TFileStorageType s_types[] = {VOLATILE, DURABLE, PERMANENT};
 	char *targetspacetoken;
 	const char srmfunc[] = "BringOnline";
 
-
+	srm_context_soap_init(context);
 
 	// issue "bringonline" request
 	memset(output,0,sizeof(*output));
 	memset (&req, 0, sizeof(req));
 
 	if ((req.arrayOfFileRequests =
-				soap_malloc (soap, sizeof(struct srm2__ArrayOfTGetFileRequest))) == NULL ||
+				soap_malloc (context->soap, sizeof(struct srm2__ArrayOfTGetFileRequest))) == NULL ||
 			(req.arrayOfFileRequests->requestArray =
-			 soap_malloc (soap, input->nbfiles * sizeof(struct srm2__TGetFileRequest *))) == NULL ||
+			 soap_malloc (context->soap, input->nbfiles * sizeof(struct srm2__TGetFileRequest *))) == NULL ||
 			(req.transferParameters =
-			 soap_malloc (soap, sizeof(struct srm2__TTransferParameters))) == NULL ||
-			(req.targetSpaceToken =
-			 soap_malloc (soap, sizeof(char *))) == NULL) {
+			 soap_malloc (context->soap, sizeof(struct srm2__TTransferParameters))) == NULL) {
 
 		srm_errmsg (context, "[SRM][soap_malloc][] error");
 		errno = ENOMEM;
-		srm_soap_free(soap);
 		return (-1);
 	}
 
@@ -850,7 +804,6 @@ int srmv2_bring_online_async_internal (struct srm_context *context,
 			req.targetSpaceToken = targetspacetoken;
 		} else
 		{
-			srm_soap_free(soap);
 			return (-1);
 		}
 	}
@@ -861,15 +814,19 @@ int srmv2_bring_online_async_internal (struct srm_context *context,
 	req.desiredFileStorageType = &s_types[PERMANENT];
 	req.targetFileRetentionPolicyInfo = NULL;
 	req.deferredStartTime = NULL;
+	if (input->desiredpintime > 0)
+		req.desiredLifeTime = &input->desiredpintime;
+
+	if (context->ext && context->ext->turl_resolution_timeout)
+		req.desiredTotalRequestTime = &(context->ext->turl_resolution_timeout);
 
 	req.arrayOfFileRequests->__sizerequestArray = input->nbfiles;
 
 	for (i = 0; i < input->nbfiles; i++) {
 		if ((req.arrayOfFileRequests->requestArray[i] =
-				soap_malloc (soap, sizeof(struct srm2__TGetFileRequest))) == NULL) {
+				soap_malloc (context->soap, sizeof(struct srm2__TGetFileRequest))) == NULL) {
 			srm_errmsg (context, "[SRM][soap_malloc][] error");
 			errno = ENOMEM;
-			srm_soap_free(soap);
 			return (-1);
 		}
 		memset (req.arrayOfFileRequests->requestArray[i], 0, sizeof(struct srm2__TGetFileRequest));
@@ -881,23 +838,21 @@ int srmv2_bring_online_async_internal (struct srm_context *context,
 	req.transferParameters->connectionType = NULL;
 	req.transferParameters->arrayOfClientNetworks = NULL;
 
-	ret = srm_set_protocol_in_transferParameters(context,soap, req.transferParameters,input->protocols);
+	ret = srm_set_protocol_in_transferParameters(context, req.transferParameters,input->protocols);
 
 	if (ret)
 	{
-		srm_soap_free(soap);
 		return (-1);
 	}
 	do
 	{
-		ret = call_function.call_srm2__srmBringOnline (soap, context->srm_endpoint, srmfunc, &req, &rep);
+		ret = call_function.call_srm2__srmBringOnline (context->soap, context->srm_endpoint, srmfunc, &req, &rep);
 		// If no response break with failure
 		if ((rep.srmBringOnlineResponse == NULL)||(ret!=0)||
 				copy_returnstatus(&output->retstatus,rep.srmBringOnlineResponse->returnStatus))
 		{
-			errno = srm_soap_call_err(context,soap,srmfunc);
+			errno = srm_soap_call_err(context, srmfunc);
 			internal_context->current_status = srm_call_status_FAILURE;
-			srm_soap_free(soap);
 			return -1;
 		}
 
@@ -958,13 +913,12 @@ int srmv2_bring_online_async_internal (struct srm_context *context,
 			break;
 	}
 
-	srm_soap_free(soap);
 	return ret;
 }
 
 
-
-int srmv2_bringonline_status_estimated_wait_time(struct srm2__srmStatusOfBringOnlineRequestResponse_ *bringOnlineStatusRep){
+int srmv2_bringonline_status_estimated_wait_time(struct srm2__srmStatusOfBringOnlineRequestResponse_ *bringOnlineStatusRep)
+{
     if( bringOnlineStatusRep && bringOnlineStatusRep->srmStatusOfBringOnlineRequestResponse && bringOnlineStatusRep->srmStatusOfBringOnlineRequestResponse->arrayOfFileStatuses
          && bringOnlineStatusRep->srmStatusOfBringOnlineRequestResponse->arrayOfFileStatuses->__sizestatusArray > 0
          && bringOnlineStatusRep->srmStatusOfBringOnlineRequestResponse->arrayOfFileStatuses->statusArray){
@@ -984,12 +938,11 @@ int srmv2_status_of_bring_online_async_internal (struct srm_context *context,
 {
 	int ret;
 	struct srm2__ArrayOfTBringOnlineRequestFileStatus *repfs;
-    struct soap* soap = srm_soap_init_context_new(context);
 	struct srm2__srmStatusOfBringOnlineRequestResponse_ srep;
 	struct srm2__srmStatusOfBringOnlineRequestRequest sreq;
 	const char srmfunc[] = "StatusOfBringOnlineRequest";
 
-
+	srm_context_soap_init(context);
 
 	memset (&sreq, 0, sizeof(sreq));
 	sreq.requestToken = (char *) output->token;
@@ -999,14 +952,13 @@ int srmv2_status_of_bring_online_async_internal (struct srm_context *context,
 
 	do
 	{
-		ret = call_function.call_srm2__srmStatusOfBringOnlineRequest(soap, context->srm_endpoint, srmfunc, &sreq, &srep);
+		ret = call_function.call_srm2__srmStatusOfBringOnlineRequest(context->soap, context->srm_endpoint, srmfunc, &sreq, &srep);
 		// If no response break with failure
 		if ((srep.srmStatusOfBringOnlineRequestResponse == NULL)||(ret!=0)||
 				copy_returnstatus(&output->retstatus,srep.srmStatusOfBringOnlineRequestResponse->returnStatus))
 		{
-			errno = srm_soap_call_err(context,soap,srmfunc);
+			errno = srm_soap_call_err(context, srmfunc);
 			internal_context->current_status = srm_call_status_FAILURE;
-			srm_soap_free(soap);
 			return -1;
 		}
         set_estimated_wait_time(internal_context, srmv2_bringonline_status_estimated_wait_time(&srep));
@@ -1060,7 +1012,6 @@ int srmv2_status_of_bring_online_async_internal (struct srm_context *context,
 			break;
 	}
 
-	srm_soap_free(soap);
 	return (ret);
 
 }
@@ -1073,24 +1024,22 @@ int srmv2_abort_files(struct srm_context *context,
 	struct srm2__ArrayOfTSURLReturnStatus *repfs;
 	struct srm2__srmAbortFilesRequest req;
 	struct srm2__TReturnStatus *reqstatp;
-    struct soap* soap = srm_soap_init_context_new(context);
 	const char srmfunc[] = "AbortFiles";
 
+	srm_context_soap_init(context);
+
 	// Setup the timeout
 	back_off_logic_init(context,&internal_context);
 
-	
-
 	memset (&req, 0, sizeof(req));
 
 	req.requestToken = (char *) input->reqtoken;
 
 	// NOTE: only one SURL in the array
 	if ((req.arrayOfSURLs =
-				soap_malloc (soap, sizeof(struct srm2__ArrayOfAnyURI))) == NULL) {
+				soap_malloc (context->soap, sizeof(struct srm2__ArrayOfAnyURI))) == NULL) {
 		srm_errmsg (context, "[SRM][soap_malloc][] error");
 		errno = ENOMEM;
-		srm_soap_free(soap);
 		return (-1);
 	}
 
@@ -1099,13 +1048,12 @@ int srmv2_abort_files(struct srm_context *context,
 
 	do
 	{
-		ret = call_function.call_srm2__srmAbortFiles (soap, context->srm_endpoint, srmfunc, &req, &rep);
+		ret = call_function.call_srm2__srmAbortFiles (context->soap, context->srm_endpoint, srmfunc, &req, &rep);
 		// If no response break with failure
 		if ((rep.srmAbortFilesResponse == NULL)||(ret!=0))
 		{
-			errno = srm_soap_call_err(context,soap,srmfunc);
+			errno = srm_soap_call_err(context, srmfunc);
 			internal_context.current_status = srm_call_status_FAILURE;
-			srm_soap_free(soap);
 			return -1;
 		}
 		// Check status and wait with back off logic if necessary(Internal_error)
@@ -1123,7 +1071,6 @@ int srmv2_abort_files(struct srm_context *context,
 			if (!repfs || repfs->__sizestatusArray < 1 || !repfs->statusArray)
 			{
 				errno = srm_call_err(context,reqstatp,srmfunc);
-				srm_soap_free(soap);
 				return (-1);
 			}else
 			{
@@ -1141,7 +1088,6 @@ int srmv2_abort_files(struct srm_context *context,
 			break;
 	}
 
-	srm_soap_free(soap);
 	return (ret);
 }
 
@@ -1150,9 +1096,9 @@ int srmv2_abort_request(struct srm_context *context,char *token)
 	const char srmfunc[] = "AbortRequest";
 	struct srm2__srmAbortRequestRequest abortreq;
 	struct srm2__srmAbortRequestResponse_ abortrep;
-    struct soap* soap = srm_soap_init_context_new(context);
 	int result;
 
+	srm_context_soap_init(context);
 
 	memset (&abortreq, 0, sizeof(abortreq));
 
@@ -1160,18 +1106,17 @@ int srmv2_abort_request(struct srm_context *context,char *token)
 	{
 		// No token supplied
 		errno = EINVAL;
-		srm_soap_free(soap);
 		return (-1);
 	}else
 	{
 		abortreq.requestToken = token;
 
-		result = call_function.call_srm2__srmAbortRequest (soap, context->srm_endpoint, srmfunc, &abortreq, &abortrep);
+		result = call_function.call_srm2__srmAbortRequest (context->soap, context->srm_endpoint, srmfunc, &abortreq, &abortrep);
 
 		if (result != 0)
 		{
 			// Soap call failure
-			errno = srm_soap_call_err(context,soap,srmfunc);
+			errno = srm_soap_call_err(context, srmfunc);
 		}else
 		{
 			if (abortrep.srmAbortRequestResponse == NULL ||
@@ -1179,13 +1124,10 @@ int srmv2_abort_request(struct srm_context *context,char *token)
 					abortrep.srmAbortRequestResponse->returnStatus->statusCode != SRM_USCORESUCCESS)
 			{
 				errno = EINVAL;
-				srm_soap_free(soap);
 				return (-1);
 			}
 		}
 	}
 
-	srm_soap_free(soap);
-
 	return result;
 }
